﻿using Hotline.Share.Enums.FlowEngine;
using System.Security.Authentication;

namespace Hotline.Share.Dtos.FlowEngine;

public class PreviousWorkflowDto : EndWorkflowIdDto
{
    /// <summary>
    /// 是否短信通知
    /// </summary>
    public bool AcceptSms { get; set; }

    /// <summary>
    /// 期满时间
    /// </summary>
    public DateTime? ExpiredTime { get; set; }

    ///// <summary>
    ///// 指定办理对象
    ///// </summary>
    //public FlowStepHandler? Handler { get; set; }

    /// <summary>
    /// 逆向流程，节点指派方式
    /// 退回、特提
    /// </summary>
    public ReverseFlowStepAssignInfo ReverseFlowStepAssignInfo { get; set; }
}

/// <summary>
/// 逆向流程，节点指派方式
/// </summary>
public class ReverseFlowStepAssignInfo
{
    public ReverseFlowStepAssignInfo()
    {
        
    }

    public ReverseFlowStepAssignInfo(EReverseFlowStepCreationPolicy reverseFlowStepCreationPolicy, StepAssignInfo? stepAssignInfo = null)
    {
        ReverseFlowStepCreationPolicy = reverseFlowStepCreationPolicy;
        StepAssignInfo = stepAssignInfo;
    }

    public EReverseFlowStepCreationPolicy ReverseFlowStepCreationPolicy { get; set; } =
        EReverseFlowStepCreationPolicy.OriginStep;

    /// <summary>
    /// 节点指定办理对象(非指定办理对象时无效)
    /// </summary>
    public StepAssignInfo? StepAssignInfo { get; set; }
}

/// <summary>
/// 逆向流程，节点创建策略
/// </summary>
public enum EReverseFlowStepCreationPolicy
{
    /// <summary>
    /// 保持原节点的办理对象不变
    /// </summary>
    OriginStep = 1,

    /// <summary>
    /// 指派给原节点办理人
    /// </summary>
    OriginStepUser = 2,

    /// <summary>
    /// 指派给原节点办理部门
    /// </summary>
    OriginStepOrg = 3,

    /// <summary>
    /// 指派给原节点办理角色
    /// </summary>
    OriginStepRole = 4,

    /// <summary>
    /// 指派给原节点办理部门及角色
    /// </summary>
    OriginStepOrgAndRole = 5,

    /// <summary>
    /// 指定办理对象
    /// </summary>
    AssignHandler = 20,

    /// <summary>
    /// 保持原节点配置(指派给角色)
    /// </summary>
    OriginDefinition = 30,
}

public class StepAssignInfo : FlowStepHandler
{
    public EFlowAssignType FlowAssignType { get; set; }
}

/// <summary>
/// 操作人信息
/// </summary>
public class OperatorInfo
{
    public OperatorInfo()
    {

    }

    public OperatorInfo(
        string? userId, string? userName,
        string? orgId, string? orgName,
        string? orgAreaCode, string orgAreaName,
        bool orgIsCenter, string[] roles,
        int orglevel)
    {
        UserId = userId;
        UserName = userName;
        OrgId = orgId;
        OrgName = orgName;
        OrgLevel = orglevel;
        OrgIsCenter = orgIsCenter;
        OrgAreaCode = orgAreaCode;
        Roles = roles;
    }

    public string? UserId { get; init; }

    public string? UserName { get; init; }

    public string? OrgId { get; init; }

    public string? OrgName { get; init; }
    public int OrgLevel { get; init; }
    public bool OrgIsCenter { get; init; }

    public string? OrgAreaCode { get; init; }

    /// <summary>
    /// 部门行政区划名称
    /// </summary>
    public string? OrgAreaName { get; init; }

    /// <summary>
    /// Roles
    /// </summary>
    public string[] Roles { get; init; }

}